@@ -15,9 +15,9 @@ from collections import namedtuple
 # Lazy loading
 from . import local
 
-SERIES = 'opalescent'
+SERIES = 'recovery'
 
-__version__ = '5.4.0'
+__version__ = '5.6.0'
 __author__ = 'Ask Solem'
 __contact__ = 'auvipy@gmail.com'
 __homepage__ = 'https://docs.celeryq.dev/'
@@ -42,7 +42,7 @@ version_info_t = namedtuple('version_info_t', (
 # bumpversion can only search for {current_version}
 # so we have to parse the version here.
 _temp = re.match(
-    r'(\d+)\.(\d+).(\d+)(.+)?', __version__).groups()
+    r'(\d+)\.(\d+)\.(\d+)(.+)?', __version__).groups()
 VERSION = version_info = version_info_t(
     int(_temp[0]), int(_temp[1]), int(_temp[2]), _temp[3] or '', '')
 del _temp
