@@ -63,13 +63,18 @@ def republish(producer, message, exchange=None, routing_key=None,
     # when the message is recompressed.
     compression = headers.pop('compression', None)
 
+    expiration = props.pop('expiration', None)
+    # ensure expiration is a float
+    expiration = float(expiration) if expiration is not None else None
+
     for key in remove_props:
         props.pop(key, None)
 
     producer.publish(ensure_bytes(body), exchange=exchange,
                      routing_key=routing_key, compression=compression,
                      headers=headers, content_type=ctype,
-                     content_encoding=enc, **props)
+                     content_encoding=enc, expiration=expiration,
+                     **props)
 
 
 def migrate_task(producer, body_, message, queues=None):
