@@ -1,15 +1,17 @@
-from __future__ import absolute_import, unicode_literals
-
 import os
 
-from case import skip
+import pytest
 
 from celery import states
 from celery.backends.azureblockblob import AzureBlockBlobBackend
 
+pytest.importorskip('azure')
 
-@skip.unless_module("azure")
-@skip.unless_environ("AZUREBLOCKBLOB_URL")
+
+@pytest.mark.skipif(
+    not os.environ.get('AZUREBLOCKBLOB_URL'),
+    reason='Environment variable AZUREBLOCKBLOB_URL required'
+)
 class test_AzureBlockBlobBackend:
     def test_crud(self, manager):
         backend = AzureBlockBlobBackend(
