@@ -1,11 +1,9 @@
-from __future__ import absolute_import, unicode_literals
-
 from celery.contrib.abortable import AbortableAsyncResult, AbortableTask
 
 
 class test_AbortableTask:
 
-    def setup(self):
+    def setup_method(self):
         @self.app.task(base=AbortableTask, shared=False)
         def abortable():
             return True
