@@ -1,11 +1,13 @@
+import os
 from base64 import b64encode
 from tempfile import NamedTemporaryFile
+from unittest.mock import patch
 
 import pytest
 
 import t.skip
 from celery.utils import term
-from celery.utils.term import _read_as_base64, colored, fg
+from celery.utils.term import _read_as_base64, colored, fg, supports_images
 
 
 @t.skip.if_win32
@@ -70,3 +72,16 @@ class test_colored:
             expected_result = b64encode(test_data).decode('ascii')
 
             assert result == expected_result
+
+    @pytest.mark.parametrize('is_tty, iterm_profile, expected', [
+        (True, 'test_profile', True),
+        (False, 'test_profile', False),
+        (True, None, False),
+    ])
+    @patch('sys.stdin.isatty')
+    @patch.dict(os.environ, {'ITERM_PROFILE': 'test_profile'}, clear=True)
+    def test_supports_images(self, mock_isatty, is_tty, iterm_profile, expected):
+        mock_isatty.return_value = is_tty
+        if iterm_profile is None:
+            del os.environ['ITERM_PROFILE']
+        assert supports_images() == expected
